@@ -1,5 +1,5 @@
 /* Lzip - LZMA lossless data compressor
-   Copyright (C) 2008-2022 Antonio Diaz Diaz.
+   Copyright (C) 2008-2024 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,12 +48,13 @@ void list_line( const unsigned long long uncomp_size,
 
 
 int list_files( const std::vector< std::string > & filenames,
-                const bool ignore_trailing, const bool loose_trailing )
+                const Cl_options & cl_opts )
   {
   unsigned long long total_comp = 0, total_uncomp = 0;
   int files = 0, retval = 0;
   bool first_post = true;
   bool stdin_used = false;
+
   for( unsigned i = 0; i < filenames.size(); ++i )
     {
     const bool from_stdin = ( filenames[i] == "-" );
@@ -65,7 +66,7 @@ int list_files( const std::vector< std::string > & filenames,
       open_instream( input_filename, &in_stats, false, true );
     if( infd < 0 ) { set_retval( retval, 1 ); continue; }
 
-    const Lzip_index lzip_index( infd, ignore_trailing, loose_trailing );
+    const Lzip_index lzip_index( infd, cl_opts );
     close( infd );
     if( lzip_index.retval() != 0 )
       {
